package de.fraunhofer.iem.dataprovider.toolResult

import de.fraunhofer.iem.dataprovider.gitlab.Repository
import jakarta.persistence.*
import org.hibernate.Hibernate
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*

enum class Level {
    Warning, Error, Note, None
}

enum class Kind {
    pass, open, informational, notApplicable
}

@Entity
@Table(name = "tool_run")
class ToolRun {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null


    @OneToOne(orphanRemoval = true, cascade = [CascadeType.ALL])
    @JoinColumn(name = "tool_id")
    var tool: Tool? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "time_stamp")
    var timeStamp: Timestamp? = null

    @ManyToOne(fetch = FetchType.EAGER)
    var repository: Repository? = null

    @OneToMany(mappedBy = "toolRun", cascade = [CascadeType.ALL], orphanRemoval = true)
    var toolResults: MutableSet<ToolResult> = mutableSetOf()

    fun addToolResults(toolResults: Collection<ToolResult>) {
        toolResults.forEach {
            this.addToolResult(it)
        }
    }

    fun addToolResult(toolResult: ToolResult) {
        toolResults.add(toolResult)
        toolResult.toolRun = this
    }
}

@Entity
@Table(name = "tool")
class Tool {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null


    @Column(name = "name", length = 500)
    var name: String? = null

    @Column(name = "version")
    var version: String? = null

    @OneToMany(mappedBy = "tool", cascade = [CascadeType.ALL], orphanRemoval = true)
    var rules: MutableSet<Rule> = mutableSetOf()

    fun addRules(rules: Collection<Rule>) {
        rules.forEach {
            this.addRule(it)
        }
    }

    fun addRule(rule: Rule) {
        rules.add(rule)
        rule.tool = this
    }
}


@Entity
@Table(name = "tool_result")
class ToolResult {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    var rule: Rule? = null

    @OneToMany(mappedBy = "toolResult", cascade = [CascadeType.ALL], orphanRemoval = true)
    var resultLocations: MutableSet<ResultLocation> = mutableSetOf()

    fun addLocations(locations: Collection<ResultLocation>) {
        locations.forEach {
            this.addLocation(it)
        }
    }

    fun addLocation(location: ResultLocation) {
        resultLocations.add(location)
        location.toolResult = this
    }

    @Enumerated
    @Column(name = "level")
    var level: Level? = null

    @Enumerated
    @Column(name = "kind")
    var kind: Kind? = null

    @Lob
    @Column(name = "message")
    var message: String? = null

    @ManyToOne
    var toolRun: ToolRun? = null

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as ToolResult

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}

@Entity
@Table(name = "result_location")
class ResultLocation {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    var toolResult: ToolResult? = null

    @Lob
    @Column(name = "artifact_location")
    var artifactLocation: String? = null

    @Column(name = "end_column")
    var endColumn: Int? = null

    @Column(name = "end_line")
    var endLine: Int? = null

    @Column(name = "start_column")
    var startColumn: Int? = null

    @Column(name = "start_line")
    var startLine: Int? = null

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as ResultLocation

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}

@Entity
@Table(name = "rule")
class Rule {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null


    @Column(name = "sarif_rule_id")
    var sarifRuleId: String? = null

    @Lob
    @Column(name = "short_description")
    var shortDescription: String? = null

    @OneToMany(mappedBy = "rule", cascade = [CascadeType.ALL], orphanRemoval = true)
    var toolResults: MutableSet<ToolResult> = mutableSetOf()

    fun addResultLocations(toolResults: Collection<ToolResult>) {
        toolResults.forEach {
            this.addResultLocation(it)
        }
    }

    fun addResultLocation(toolResult: ToolResult) {
        toolResults.add(toolResult)
        toolResult.rule = this
    }

    @ElementCollection
    @CollectionTable(name = "rule_messages", joinColumns = [JoinColumn(name = "owner_id")])
    @Column(name = "message", length = 1000)
    var messages: MutableList<String> = mutableListOf()

    @ManyToOne
    var tool: Tool? = null

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as Rule

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}